/* Copyright 1998 Donald J. Bindner */

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <conio.h>
#include <stat.h>
#include <process.h>
#include <string.h>
#include <ctype.h>

#define EVEN 0
#define ODD 1
#define ALL 2
#define STDOUT 1

void usage( int argc, char **argv ) {
    printf( "Duplex utility for Sprint\n" );
    printf( "Usage is:\n" );
    printf( "\tduplex {options} <file>\n" );
    printf( "options:\n" );
    printf( "\t-R\treverse page order\n" );
    printf( "\t-O\todd pages only\n" );
    printf( "\t-E\teven pages only\n" );
    printf( "other options:\n" );
    printf( "\t-dname{=value}\tdefine variable 'name'\n" );
    printf( "\t-f=name\t\tuse alternate font\n" );
    printf( "\t-o{=name}\tsend output to name.PRN\n" );
    printf( "\t-p=name\t\tuse alternate printer\n" );
    printf( "\t-pause\t\tpause to insert pages into the printer\n" );
    printf( "\t-page=#\t\tprint only page #\n" );
    printf( "\t-start=#\tstart printing with page #\n" );
    printf( "\t-stop=#\t\tstop printing after page #\n" );
    printf( "\t-s{=name}\tstatus line on\n" );
    printf( "\t-t=#\t\tset default tab stop every # characters\n" );
    printf( "\t-v\t\tpreview formatted output\n" );
    printf( "\t-z\t\tdon't know\n" );

    exit( 2 );
}

void do_pause( int pause ) {
    while( getkey() != EOF );
    
    if( pause ) {
	fprintf( stderr, "[Place a new page in the printer, type any key when ready, Esc to quit]\n" );
	if( getch() == (char)27 ) {
	    exit( 4 );
	}
    }
}

void fgetln( char *line, int size, FILE *f ) {
    char *p = line;
    int ch;
    int i = 0;

    while(( i < size-1 ) && (( ch = fgetc( f )) != EOF )) {
        *p = (char)ch;
        p++;
        i++;
        if(( ch == 13 ) || ( ch == 10 )) break;
    }
    *p = 0;
}

void update( char *line, int *lastpage ) {
    char *p = line;
    int page;

    /* skip over filename */
    while( !isspace( *p )) p++;

    /* skip over white space */
    while( isspace( *p )) p++;

    /* first syntax check */
    if( strncmp( p, "line", 4 )) {
        return;
    } else p += 4;

    /* skip over line number */
    while( isspace( *p ) || isdigit( *p )) p++;

    /* 2nd syntax check */
    if( strncmp( p, "Page", 4 )) {
        return;
    } else p += 4;

    page = atoi( p );
    if( page > *lastpage ) *lastpage = page;
}

int main( int argc, char *argv[] ) {
    FILE *f;
/*    FILE *n; */
/*    int dupOutFH=0; */
    int reverse=0, pause=0;
    int rule=ALL;
    char *fname;
    char pagestr[256];
    char line[256];
    struct stat sbuff;
    int i, err;
    int firstpage=1, lastpage=0;
    char *srule[] = { "even", "odd", "all" };
    char *srev[] = { ".", " reversed." };
    char *sp_arg[20];
    int count;

    count = 0;
    sp_arg[count] = "SPFMT";	count++;
    
    /* Parse the command line arguments */
    for( i = 1; i < argc; i++ ) {
	/* Our flags first */
        if( !strcmp( argv[i], "-R" )) {
            reverse = 1;
        } else if( !strcmp( argv[i], "-O" )) {
            rule = ODD;
        } else if( !strcmp( argv[i], "-E" )) {
            rule = EVEN;
	/* Now the regular spfmt flags */
	} else if( !strncmp( argv[i], "-d", 2 )) {
	    sp_arg[count] = strdup( argv[i] );
	    count++;
	} else if( !strncmp( argv[i], "-f=", 3 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
	} else if( !strncmp( argv[i], "-o", 2 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
	} else if( !strncmp( argv[i], "-p=", 3 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
	} else if( !strncmp( argv[i], "-pause", 6 )) {
	    pause = 1;
	} else if( !strncmp( argv[i], "-page=", 6 )) {
	    firstpage = lastpage = atoi( argv[i]+6 );
	} else if( !strncmp( argv[i], "-start=", 7 )) {
	    firstpage = atoi( argv[i]+7 );
	} else if( !strncmp( argv[i], "-stop=", 6 )) {
	    lastpage = atoi( argv[i]+6 );
	} else if( !strncmp( argv[i], "-s", 2 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
	} else if( !strncmp( argv[i], "-t=", 3 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
	} else if( !strncmp( argv[i], "-v", 2 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
	} else if( !strncmp( argv[i], "-z", 2 )) {
	    sp_arg[count] = strdup( argv[i]);
	    count++;
        } else if( fname == NULL ) {
            fname = strdup( argv[i] );
        } else usage( argc, argv );
    }

    /* Print help if no file was specified */
    if( fname == NULL ) usage( argc, argv );

    /* Print help if the file doesn't exist */
    if( stat( fname, &sbuff )) {
	printf( "Error: Couldn't find %s!\n", fname );
	usage( argc, argv );
    }

    sp_arg[count] = "-l=LOG.$$$";	count++;
    sp_arg[count] = "-p0";		count++;
    sp_arg[count] = fname;		count++;
    sp_arg[count] = NULL;

    /* Get page information by calling spfmt */
    if( lastpage < firstpage ) {
	err = spawnvp( P_WAIT, "SPFMT", sp_arg );
	if( err == -1 ) {
	    printf( "Unable to execute spfmt (check your PATH).\n" );
	    exit( 3 );
	}

	if(( f = fopen( "LOG.$$$", "rb" )) == NULL ) {
	    printf( "Error: LOG.$$$ not found.\n" );
	    exit( 1 );
	}

	while( !feof( f )) {
	    fgetln( line, sizeof( line ), f );
	    update( line, &lastpage );
	}
	fclose( f );
	unlink( "LOG.$$$" );
    }
    /* Drop off the last sp_arg */
    count -= 3;

    /* Add the page number and filename arguments */
    sp_arg[count] = pagestr; count++;
    sp_arg[count] = fname; count++;
    sp_arg[count] = NULL;

    printf( "Printing %s pages from %d to %d%s\n", srule[rule],
        firstpage, lastpage, srev[reverse] );

    /* Send standard out to NUL to suppress messages */
    /* Can't redirect stdout or stuff breaks so we comment this out */
    /* if(( dupOutFH = dup( STDOUT )) == -1 ) {
	dupOutFH = 0;
    } else {
        if((n = fopen( "NUL", "wb" )) != NULL )
            dup2( n->fd, STDOUT );
    } */

    if( reverse ) {
        for( i = lastpage; i >= firstpage; i-- ) {
            if(( rule == ODD ) && !( i % 2 )) continue;
            if(( rule == EVEN ) && ( i % 2 )) continue;

	    do_pause( pause );

            sprintf( pagestr, "-page=%d", i );
	    spawnvp( P_WAIT, "SPFMT", sp_arg );
        }
    } else {
        for( i = firstpage; i <= lastpage; i++ ) {
            if(( rule == ODD ) && !( i % 2 )) continue;
            if(( rule == EVEN ) && ( i % 2 )) continue;

	    do_pause( pause );

            sprintf( pagestr, "-page=%d", i );
	    spawnvp( P_WAIT, "SPFMT", sp_arg );
        }
    }
    /* Restore the standard out if it was modified */
    /* if( dupOutFH != 0 ) dup2( dupOutFH, STDOUT ); */
}
